@@ -5,7 +5,7 @@ import numpy as np
 import tensorflow as tf
 from bert4keras.backend import keras, K, is_tf_keras
 from bert4keras.snippets import is_string, string_matching
-from bert4keras.snippets import is_one_of
+from bert4keras.snippets import is_one_of, insert_arguments
 from bert4keras.backend import piecewise_linear
 import re
 
@@ -14,15 +14,17 @@ class Adam(keras.optimizers.Optimizer):
     """重新定义Adam优化器，便于派生出新的优化器
     （tensorflow的optimizer_v2类）
     """
-    def __init__(self,
-                 learning_rate=0.001,
-                 beta_1=0.9,
-                 beta_2=0.999,
-                 epsilon=1e-6,
-                 bias_correction=True,
-                 name='Adam',
-                 **kwargs):
-        super(Adam, self).__init__(name=name, **kwargs)
+    def __init__(
+        self,
+        learning_rate=0.001,
+        beta_1=0.9,
+        beta_2=0.999,
+        epsilon=1e-6,
+        bias_correction=True,
+        **kwargs
+    ):
+        kwargs['name'] = kwargs.get('name') or 'Adam'
+        super(Adam, self).__init__(**kwargs)
         self._set_hyper('learning_rate', learning_rate)
         self._set_hyper('beta_1', beta_1)
         self._set_hyper('beta_2', beta_2)
@@ -34,7 +36,7 @@ class Adam(keras.optimizers.Optimizer):
             self.add_slot(var, 'm')
             self.add_slot(var, 'v')
 
-    def _resource_apply_op(self, grad, var, indices=None):
+    def _resource_apply(self, grad, var, indices=None):
         # 准备变量
         var_dtype = var.dtype.base_dtype
         lr_t = self._decayed_lr(var_dtype)
@@ -54,24 +56,26 @@ class Adam(keras.optimizers.Optimizer):
         else:
             mv_ops = [K.update(m, beta_1_t * m), K.update(v, beta_2_t * v)]
             with tf.control_dependencies(mv_ops):
-                m_t = self._resource_scatter_add(m, indices,
-                                                 (1 - beta_1_t) * grad)
-                v_t = self._resource_scatter_add(v, indices,
-                                                 (1 - beta_2_t) * grad**2)
+                m_t = self._resource_scatter_add(
+                    m, indices, (1 - beta_1_t) * grad
+                )
+                v_t = self._resource_scatter_add(
+                    v, indices, (1 - beta_2_t) * grad**2
+                )
 
         # 返回算子
         with tf.control_dependencies([m_t, v_t]):
             if self.bias_correction:
-                m_t = m_t / (1. - beta_1_t_power)
-                v_t = v_t / (1. - beta_2_t_power)
+                m_t = m_t / (1.0 - beta_1_t_power)
+                v_t = v_t / (1.0 - beta_2_t_power)
             var_t = var - lr_t * m_t / (K.sqrt(v_t) + self.epsilon)
             return K.update(var, var_t)
 
     def _resource_apply_dense(self, grad, var):
-        return self._resource_apply_op(grad, var)
+        return self._resource_apply(grad, var)
 
     def _resource_apply_sparse(self, grad, var, indices):
-        return self._resource_apply_op(grad, var, indices)
+        return self._resource_apply(grad, var, indices)
 
     def get_config(self):
         config = {
@@ -90,16 +94,17 @@ class AdaFactorBase(keras.optimizers.Optimizer):
     参考实现：https://github.com/tensorflow/mesh/blob/master/mesh_tensorflow/optimize.py
     """
     def __init__(
-            self,
-            learning_rate=1e-3,  # 可以为None
-            beta1=0.0,
-            beta2=None,
-            epsilon1=1e-30,
-            epsilon2=1e-3,
-            multiply_by_parameter_scale=True,
-            clipping_threshold=1.0,
-            min_dim_size_to_factor=128,
-            **kwargs):
+        self,
+        learning_rate=1e-3,  # 可以为None
+        beta1=0.0,
+        beta2=None,
+        epsilon1=1e-30,
+        epsilon2=1e-3,
+        multiply_by_parameter_scale=True,
+        clipping_threshold=1.0,
+        min_dim_size_to_factor=128,
+        **kwargs
+    ):
         super(AdaFactorBase, self).__init__(**kwargs)
         self._learning_rate = learning_rate
         self.beta1 = beta1
@@ -122,8 +127,9 @@ class AdaFactorBase(keras.optimizers.Optimizer):
         else:
             if not hasattr(self, '__learning_rate'):
                 with K.name_scope(self.__class__.__name__):
-                    self.__learning_rate = K.variable(self._learning_rate,
-                                                      name='learning_rate')
+                    self.__learning_rate = K.variable(
+                        self._learning_rate, name='learning_rate'
+                    )
             return self.__learning_rate
 
     @property
@@ -232,8 +238,7 @@ class AdaFactorV2(AdaFactorBase):
     参考实现：https://github.com/tensorflow/mesh/blob/master/mesh_tensorflow/optimize.py
     """
     def __init__(self, *args, **kwargs):
-        if 'name' not in kwargs:
-            kwargs['name'] = 'AdaFactor'
+        kwargs['name'] = kwargs.get('name') or 'AdaFactor'
         super(AdaFactorV2, self).__init__(*args, **kwargs)
 
     def _create_slots(self, var_list):
@@ -250,7 +255,7 @@ class AdaFactorV2(AdaFactorBase):
                 self.add_slot(var, 'vr', value1)
                 self.add_slot(var, 'vc', value2)
 
-    def _resource_apply_dense(self, grad, var):
+    def _resource_apply(self, grad, var, indices=None):
         lr = self.learning_rate
         g2 = K.square(grad) + self.epsilon1
         shape = K.int_shape(var)
@@ -289,6 +294,9 @@ class AdaFactorV2(AdaFactorBase):
         # 更新参数
         return K.update(var, var - lr * u)
 
+    def _resource_apply_dense(self, grad, var):
+        return self._resource_apply(grad, var)
+
     def _resource_apply_sparse(self, grad, var, indices):
         grad = tf.IndexedSlices(grad, indices, K.shape(var))
         grad = tf.convert_to_tensor(grad)
@@ -298,35 +306,30 @@ class AdaFactorV2(AdaFactorBase):
 def export_to_custom_objects(base_extend_with):
     """装饰器，用来将优化器放到custom_objects中
     """
-    def new_extend_with(base_optimizer, name=None):
-        new_optimizer = base_extend_with(base_optimizer)
+    def new_extend_with(BaseOptimizer, name=None):
+        NewOptimizer = base_extend_with(BaseOptimizer)
 
         if is_string(name):
-            new_optimizer.__name__ = name
+            NewOptimizer.__name__ = name
 
-        name = new_optimizer.__name__
-        keras.utils.get_custom_objects()[name] = new_optimizer
+        name = NewOptimizer.__name__
+        keras.utils.get_custom_objects()[name] = NewOptimizer
 
-        return new_optimizer
+        return NewOptimizer
 
     return new_extend_with
 
 
 @export_to_custom_objects
-def extend_with_weight_decay(base_optimizer):
+def extend_with_weight_decay(BaseOptimizer):
     """返回新的优化器类，加入权重衰减
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有权重衰减的优化器
         """
-        def __init__(self,
-                     weight_decay_rate,
-                     exclude_from_weight_decay=None,
-                     *args,
-                     **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.weight_decay_rate = weight_decay_rate
-            self.exclude_from_weight_decay = exclude_from_weight_decay or []
+        @insert_arguments(weight_decay_rate=0.01, exclude_from_weight_decay=[])
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
             if not hasattr(self, 'learning_rate'):
                 self.learning_rate = self.lr
 
@@ -340,43 +343,37 @@ def extend_with_weight_decay(base_optimizer):
                 return old_update(x, new_x)
 
             K.update = new_update
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
             K.update = old_update
 
             return updates
 
         def _do_weight_decay(self, w):
-            return (not string_matching(w.name,
-                                        self.exclude_from_weight_decay))
+            return (not string_matching(w.name, self.exclude_from_weight_decay))
 
         def get_config(self):
             config = {
                 'weight_decay_rate': self.weight_decay_rate,
-                'exclude_from_weight_decay': self.exclude_from_weight_decay
+                'exclude_from_weight_decay': self.exclude_from_weight_decay,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_weight_decay_v2(base_optimizer):
+def extend_with_weight_decay_v2(BaseOptimizer):
     """返回新的优化器类，加入权重衰减
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有权重衰减的优化器
         """
-        def __init__(self,
-                     weight_decay_rate,
-                     exclude_from_weight_decay=None,
-                     *args,
-                     **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.weight_decay_rate = weight_decay_rate
-            self.exclude_from_weight_decay = exclude_from_weight_decay or []
+        @insert_arguments(weight_decay_rate=0.01, exclude_from_weight_decay=[])
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
-        def _resource_apply_op(self, grad, var, indices=None):
+        def _resource_apply(self, grad, var, indices=None):
             old_update = K.update
 
             def new_update(x, new_x):
@@ -386,40 +383,37 @@ def extend_with_weight_decay_v2(base_optimizer):
                 return old_update(x, new_x)
 
             K.update = new_update
-            op = super(new_optimizer,
-                       self)._resource_apply_op(grad, var, indices)
+            op = super(NewOptimizer, self)._resource_apply(grad, var, indices)
             K.update = old_update
 
             return op
 
         def _do_weight_decay(self, w):
-            return (not string_matching(w.name,
-                                        self.exclude_from_weight_decay))
+            return (not string_matching(w.name, self.exclude_from_weight_decay))
 
         def get_config(self):
             config = {
                 'weight_decay_rate': self.weight_decay_rate,
-                'exclude_from_weight_decay': self.exclude_from_weight_decay
+                'exclude_from_weight_decay': self.exclude_from_weight_decay,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_layer_adaptation(base_optimizer):
+def extend_with_layer_adaptation(BaseOptimizer):
     """返回新的优化器类，加入层自适应学习率
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有层自适应学习率的优化器
         用每一层参数的模长来校正当前参数的学习率
         https://arxiv.org/abs/1904.00962
         """
-        def __init__(self, exclude_from_layer_adaptation=None, *args,
-                     **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.exclude_from_layer_adaptation = exclude_from_layer_adaptation or []
+        @insert_arguments(exclude_from_layer_adaptation=[])
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
             if not hasattr(self, 'learning_rate'):
                 self.learning_rate = self.lr
 
@@ -434,47 +428,49 @@ def extend_with_layer_adaptation(base_optimizer):
                     x_norm = tf.norm(x)
                     g_norm = tf.norm(dx / lr_t)
                     ratio = K.switch(
-                        x_norm > 0.,
-                        K.switch(g_norm > K.epsilon(), x_norm / g_norm, 1.),
-                        1.)
+                        x_norm > 0.0,
+                        K.switch(g_norm > K.epsilon(), x_norm / g_norm, 1.0),
+                        1.0
+                    )
                     new_x = x + dx * ratio
                 return old_update(x, new_x)
 
             K.update = new_update
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
             K.update = old_update
 
             return updates
 
         def _do_layer_adaptation(self, w):
-            return (not string_matching(w.name,
-                                        self.exclude_from_layer_adaptation))
+            return (
+                not string_matching(w.name, self.exclude_from_layer_adaptation)
+            )
 
         def get_config(self):
             config = {
-                'exclude_from_layer_adaptation': self.exclude_from_layer_adaptation
+                'exclude_from_layer_adaptation':
+                    self.exclude_from_layer_adaptation,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_layer_adaptation_v2(base_optimizer):
+def extend_with_layer_adaptation_v2(BaseOptimizer):
     """返回新的优化器类，加入层自适应学习率
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有层自适应学习率的优化器
         用每一层参数的模长来校正当前参数的学习率
         https://arxiv.org/abs/1904.00962
         """
-        def __init__(self, exclude_from_layer_adaptation=None, *args,
-                     **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.exclude_from_layer_adaptation = exclude_from_layer_adaptation or []
+        @insert_arguments(exclude_from_layer_adaptation=[])
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
-        def _resource_apply_op(self, grad, var, indices=None):
+        def _resource_apply(self, grad, var, indices=None):
             old_update = K.update
 
             def new_update(x, new_x):
@@ -485,46 +481,49 @@ def extend_with_layer_adaptation_v2(base_optimizer):
                     x_norm = tf.norm(x)
                     g_norm = tf.norm(dx / lr_t)
                     ratio = K.switch(
-                        x_norm > 0.,
-                        K.switch(g_norm > K.epsilon(), x_norm / g_norm, 1.),
-                        1.)
+                        x_norm > 0.0,
+                        K.switch(g_norm > K.epsilon(), x_norm / g_norm, 1.0),
+                        1.0
+                    )
                     new_x = x + dx * ratio
                 return old_update(x, new_x)
 
             K.update = new_update
-            op = super(new_optimizer,
-                       self)._resource_apply_op(grad, var, indices)
+            op = super(NewOptimizer, self)._resource_apply(grad, var, indices)
             K.update = old_update
 
             return op
 
         def _do_layer_adaptation(self, w):
-            return (not string_matching(w.name,
-                                        self.exclude_from_layer_adaptation))
+            return (
+                not string_matching(w.name, self.exclude_from_layer_adaptation)
+            )
 
         def get_config(self):
             config = {
-                'exclude_from_layer_adaptation': self.exclude_from_layer_adaptation
+                'exclude_from_layer_adaptation':
+                    self.exclude_from_layer_adaptation,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_piecewise_linear_lr(base_optimizer):
+def extend_with_piecewise_linear_lr(BaseOptimizer):
     """返回新的优化器类，加入分段线性学习率
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有分段线性学习率的优化器
         其中schedule是形如{1000: 1, 2000: 0.1}的字典，
         表示0～1000步内学习率线性地从零增加到100%，然后
         1000～2000步内线性地降到10%，2000步以后保持10%
         """
-        def __init__(self, lr_schedule, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.lr_schedule = {int(i): j for i, j in lr_schedule.items()}
+        @insert_arguments(lr_schedule={0: 1})
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
+            self.lr_schedule = {int(i): j for i, j in self.lr_schedule.items()}
 
         @K.symbolic
         def get_updates(self, loss, params):
@@ -538,62 +537,67 @@ def extend_with_piecewise_linear_lr(base_optimizer):
                 return old_update(x, new_x)
 
             K.update = new_update
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
             K.update = old_update
 
             return updates
 
         def get_config(self):
-            config = {'lr_schedule': self.lr_schedule}
-            base_config = super(new_optimizer, self).get_config()
+            config = {
+                'lr_schedule': self.lr_schedule,
+            }
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_piecewise_linear_lr_v2(base_optimizer):
+def extend_with_piecewise_linear_lr_v2(BaseOptimizer):
     """返回新的优化器类，加入分段线性学习率
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有分段线性学习率的优化器
         其中schedule是形如{1000: 1, 2000: 0.1}的字典，
         表示0～1000步内学习率线性地从零增加到100%，然后
         1000～2000步内线性地降到10%，2000步以后保持10%
         """
-        def __init__(self, lr_schedule, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.lr_schedule = {int(i): j for i, j in lr_schedule.items()}
+        @insert_arguments(lr_schedule={0: 1})
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
+            self.lr_schedule = {int(i): j for i, j in self.lr_schedule.items()}
 
         def _decayed_lr(self, var_dtype):
             lr_multiplier = piecewise_linear(self.iterations, self.lr_schedule)
-            lr_t = super(new_optimizer, self)._decayed_lr(var_dtype)
+            lr_t = super(NewOptimizer, self)._decayed_lr(var_dtype)
             return lr_t * K.cast(lr_multiplier, var_dtype)
 
         def get_config(self):
-            config = {'lr_schedule': self.lr_schedule}
-            base_config = super(new_optimizer, self).get_config()
+            config = {
+                'lr_schedule': self.lr_schedule,
+            }
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_gradient_accumulation(base_optimizer):
+def extend_with_gradient_accumulation(BaseOptimizer):
     """返回新的优化器类，加入梯度累积
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有梯度累积的优化器
         """
-        def __init__(self, grad_accum_steps, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.grad_accum_steps = grad_accum_steps
+        @insert_arguments(grad_accum_steps=2)
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
             self._first_get_gradients = True
 
         def get_gradients(self, loss, params):
             if self._first_get_gradients:
                 self._first_get_gradients = False
-                return super(new_optimizer, self).get_gradients(loss, params)
+                return super(NewOptimizer, self).get_gradients(loss, params)
             else:
                 return [ag / self.grad_accum_steps for ag in self.accum_grads]
 
@@ -605,9 +609,9 @@ def extend_with_gradient_accumulation(base_optimizer):
             # 获取梯度
             grads = self.get_gradients(loss, params)
             self.accum_grads = [
-                K.zeros(K.int_shape(p),
-                        dtype=K.dtype(p),
-                        name='accum_grad_%s' % i) for i, p in enumerate(params)
+                K.zeros(
+                    K.int_shape(p), dtype=K.dtype(p), name='accum_grad_%s' % i
+                ) for i, p in enumerate(params)
             ]
 
             old_update = K.update
@@ -617,7 +621,7 @@ def extend_with_gradient_accumulation(base_optimizer):
                 return old_update(x, new_x)
 
             K.update = new_update
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
             K.update = old_update
 
             # 累积梯度
@@ -630,30 +634,32 @@ def extend_with_gradient_accumulation(base_optimizer):
             return accum_updates
 
         def get_config(self):
-            config = {'grad_accum_steps': self.grad_accum_steps}
-            base_config = super(new_optimizer, self).get_config()
+            config = {
+                'grad_accum_steps': self.grad_accum_steps,
+            }
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_gradient_accumulation_v2(base_optimizer):
+def extend_with_gradient_accumulation_v2(BaseOptimizer):
     """返回新的优化器类，加入梯度累积
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有梯度累积的优化器
         """
-        def __init__(self, grad_accum_steps, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.grad_accum_steps = grad_accum_steps
+        @insert_arguments(grad_accum_steps=2)
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
         def _create_slots(self, var_list):
-            super(new_optimizer, self)._create_slots(var_list)
+            super(NewOptimizer, self)._create_slots(var_list)
             for var in var_list:
                 self.add_slot(var, 'ag')
 
-        def _resource_apply_op(self, grad, var, indices=None):
+        def _resource_apply(self, grad, var, indices=None):
             # 更新判据
             cond = K.equal(self.iterations % self.grad_accum_steps, 0)
             # 获取梯度
@@ -667,7 +673,7 @@ def extend_with_gradient_accumulation_v2(base_optimizer):
 
             K.update = new_update
             ag_t = ag / self.grad_accum_steps
-            op = super(new_optimizer, self)._resource_apply_op(ag_t, var)
+            op = super(NewOptimizer, self)._resource_apply(ag_t, var)
             K.update = old_update
 
             # 累积梯度
@@ -682,42 +688,39 @@ def extend_with_gradient_accumulation_v2(base_optimizer):
             return ag_t
 
         def get_config(self):
-            config = {'grad_accum_steps': self.grad_accum_steps}
-            base_config = super(new_optimizer, self).get_config()
+            config = {
+                'grad_accum_steps': self.grad_accum_steps,
+            }
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_lookahead(base_optimizer):
+def extend_with_lookahead(BaseOptimizer):
     """返回新的优化器类，加入look ahead
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有look ahead的优化器
         https://arxiv.org/abs/1907.08610
         steps_per_slow_update: 即论文中的k；
         slow_step_size: 即论文中的alpha。
         """
-        def __init__(self,
-                     steps_per_slow_update=5,
-                     slow_step_size=0.5,
-                     *args,
-                     **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.steps_per_slow_update = steps_per_slow_update
-            self.slow_step_size = slow_step_size
+        @insert_arguments(steps_per_slow_update=5, slow_step_size=0.5)
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
         @K.symbolic
         def get_updates(self, loss, params):
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
 
             k, alpha = self.steps_per_slow_update, self.slow_step_size
             cond = K.equal(self.iterations % k, 0)
             slow_vars = [
-                K.zeros(K.int_shape(p),
-                        dtype=K.dtype(p),
-                        name='slow_var_%s' % i) for i, p in enumerate(params)
+                K.zeros(
+                    K.int_shape(p), dtype=K.dtype(p), name='slow_var_%s' % i
+                ) for i, p in enumerate(params)
             ]
 
             with tf.control_dependencies(updates):
@@ -736,41 +739,35 @@ def extend_with_lookahead(base_optimizer):
         def get_config(self):
             config = {
                 'steps_per_slow_update': self.steps_per_slow_update,
-                'slow_step_size': self.slow_step_size
+                'slow_step_size': self.slow_step_size,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_lookahead_v2(base_optimizer):
+def extend_with_lookahead_v2(BaseOptimizer):
     """返回新的优化器类，加入look ahead
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有look ahead的优化器
         https://arxiv.org/abs/1907.08610
         steps_per_slow_update: 即论文中的k；
         slow_step_size: 即论文中的alpha。
         """
-        def __init__(self,
-                     steps_per_slow_update=5,
-                     slow_step_size=0.5,
-                     *args,
-                     **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.steps_per_slow_update = steps_per_slow_update
-            self.slow_step_size = slow_step_size
+        @insert_arguments(steps_per_slow_update=5, slow_step_size=0.5)
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
         def _create_slots(self, var_list):
-            super(new_optimizer, self)._create_slots(var_list)
+            super(NewOptimizer, self)._create_slots(var_list)
             for var in var_list:
                 self.add_slot(var, 'slow_var')
 
-        def _resource_apply_op(self, grad, var, indices=None):
-            op = super(new_optimizer,
-                       self)._resource_apply_op(grad, var, indices)
+        def _resource_apply(self, grad, var, indices=None):
+            op = super(NewOptimizer, self)._resource_apply(grad, var, indices)
 
             k, alpha = self.steps_per_slow_update, self.slow_step_size
             cond = K.equal(self.iterations % k, 0)
@@ -778,8 +775,9 @@ def extend_with_lookahead_v2(base_optimizer):
             slow_var_t = slow_var + alpha * (var - slow_var)
 
             with tf.control_dependencies([op]):
-                slow_update = K.update(slow_var,
-                                       K.switch(cond, slow_var_t, slow_var))
+                slow_update = K.update(
+                    slow_var, K.switch(cond, slow_var_t, slow_var)
+                )
                 with tf.control_dependencies([slow_update]):
                     copy_update = K.update(var, K.switch(cond, slow_var, var))
 
@@ -788,32 +786,32 @@ def extend_with_lookahead_v2(base_optimizer):
         def get_config(self):
             config = {
                 'steps_per_slow_update': self.steps_per_slow_update,
-                'slow_step_size': self.slow_step_size
+                'slow_step_size': self.slow_step_size,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_lazy_optimization(base_optimizer):
+def extend_with_lazy_optimization(BaseOptimizer):
     """返回新的优化器类，加入懒惰更新
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有懒惰更新的优化器
         使得部分权重（尤其是embedding）只有在梯度不等于0时
         才发生更新。
         """
-        def __init__(self, include_in_lazy_optimization=None, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.include_in_lazy_optimization = include_in_lazy_optimization or []
+        @insert_arguments(include_in_lazy_optimization=[])
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
             self._first_get_gradients = True
 
         def get_gradients(self, loss, params):
             if self._first_get_gradients:
                 self._first_get_gradients = False
-                return super(new_optimizer, self).get_gradients(loss, params)
+                return super(NewOptimizer, self).get_gradients(loss, params)
             else:
                 return [self.grads[p] for p in params]
 
@@ -826,12 +824,12 @@ def extend_with_lazy_optimization(base_optimizer):
             def new_update(x, new_x):
                 if is_one_of(x, params) and self._do_lazy_optimization(x):
                     g = self.grads[x]
-                    r = K.any(K.not_equal(g, 0.), axis=-1, keepdims=True)
+                    r = K.any(K.not_equal(g, 0.0), axis=-1, keepdims=True)
                     new_x = x + (new_x - x) * K.cast(r, K.floatx())
                 return old_update(x, new_x)
 
             K.update = new_update
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
             K.update = old_update
 
             return updates
@@ -841,46 +839,47 @@ def extend_with_lazy_optimization(base_optimizer):
 
         def get_config(self):
             config = {
-                'include_in_lazy_optimization': self.include_in_lazy_optimization
+                'include_in_lazy_optimization':
+                    self.include_in_lazy_optimization,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_lazy_optimization_v2(base_optimizer):
+def extend_with_lazy_optimization_v2(BaseOptimizer):
     """返回新的优化器类，加入懒惰更新
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带有懒惰更新的优化器
         使得部分权重（尤其是embedding）只有在梯度不等于0时
         才发生更新。
         """
-        def __init__(self, include_in_lazy_optimization=None, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.include_in_lazy_optimization = include_in_lazy_optimization or []
+        @insert_arguments(include_in_lazy_optimization=[])
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
-        def _resource_apply_op(self, grad, var, indices=None):
+        def _resource_apply(self, grad, var, indices=None):
             old_update = K.update
 
             def new_update(x, new_x):
                 if x is var and self._do_lazy_optimization(x):
                     if indices is None:
-                        r = K.any(K.not_equal(grad, 0.),
-                                  axis=-1,
-                                  keepdims=True)
+                        r = K.any(
+                            K.not_equal(grad, 0.0), axis=-1, keepdims=True
+                        )
                         new_x = x + (new_x - x) * K.cast(r, K.floatx())
                         return old_update(x, new_x)
                     else:
                         return self._resource_scatter_add(
-                            x, indices, K.gather(new_x - x, indices))
+                            x, indices, K.gather(new_x - x, indices)
+                        )
                 return old_update(x, new_x)
 
             K.update = new_update
-            op = super(new_optimizer,
-                       self)._resource_apply_op(grad, var, indices)
+            op = super(NewOptimizer, self)._resource_apply(grad, var, indices)
             K.update = old_update
 
             return op
@@ -890,27 +889,28 @@ def extend_with_lazy_optimization_v2(base_optimizer):
 
         def get_config(self):
             config = {
-                'include_in_lazy_optimization': self.include_in_lazy_optimization
+                'include_in_lazy_optimization':
+                    self.include_in_lazy_optimization,
             }
-            base_config = super(new_optimizer, self).get_config()
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 @export_to_custom_objects
-def extend_with_exponential_moving_average(base_optimizer):
+def extend_with_exponential_moving_average(BaseOptimizer):
     """返回新的优化器类，加入EMA（权重滑动平均）
     """
-    class new_optimizer(base_optimizer):
+    class NewOptimizer(BaseOptimizer):
         """带EMA（权重滑动平均）的优化器
         """
-        def __init__(self, ema_momentum=0.999, *args, **kwargs):
-            super(new_optimizer, self).__init__(*args, **kwargs)
-            self.ema_momentum = ema_momentum
+        @insert_arguments(ema_momentum=0.999)
+        def __init__(self, *args, **kwargs):
+            super(NewOptimizer, self).__init__(*args, **kwargs)
 
         def get_updates(self, loss, params):
-            updates = super(new_optimizer, self).get_updates(loss, params)
+            updates = super(NewOptimizer, self).get_updates(loss, params)
             self.model_weights = params
             self.ema_weights = [K.zeros(K.shape(w)) for w in params]
             self.old_weights = K.batch_get_value(params)
@@ -925,8 +925,10 @@ def extend_with_exponential_moving_average(base_optimizer):
             return ema_updates
 
         def get_config(self):
-            config = {'ema_momentum': self.ema_momentum}
-            base_config = super(new_optimizer, self).get_config()
+            config = {
+                'ema_momentum': self.ema_momentum,
+            }
+            base_config = super(NewOptimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
         def apply_ema_weights(self):
@@ -941,7 +943,7 @@ def extend_with_exponential_moving_average(base_optimizer):
             """
             K.batch_set_value(zip(self.model_weights, self.old_weights))
 
-    return new_optimizer
+    return NewOptimizer
 
 
 if is_tf_keras:
